@@ -14,21 +14,46 @@
 #' @export
 #' @return It will return a \code{lggplot} object with class \code{c("lggplot", "gg", "ggplot")}.
 #' Then print a \code{loon} plot automatically.
+#'
+#' @seealso \code{\link{loon.ggplot}}
 #' @examples
 #' if(interactive()) {
-#'   p <- l_ggplot(mpg, aes(displ, cty)) +
-#'      geom_point() +
-#'      facet_grid(rows = vars(drv))
-#'   # p is a `lggplot` object, `print.lggplot(p)` is called automatically.
-#'   # Then, the `lggplot` object will be transformed to a `loon` widget
-#'   p
-#' }
+#' p <- l_ggplot(mpg, aes(displ, cty)) +
+#'     geom_point(
+#'       size = 4,
+#'       mapping = aes(color = factor(cyl))
+#'     )
+#' # p is a `lggplot` object, `print.lggplot(p)` will be called automatically.
+#' # Then, a `lggplot` object will be transformed to a `loon` widget
+#' p
+#'
 #' \dontrun{
-#'   # get widgets from current path
-#'   # suppose the path of `p` is '.l0.ggplot'
-#'   q <- l_getFromPath('.l0.ggplot')
-#'   # q is a `loon` widget
-#'   q
+#' # Assign a widget from current path
+#' # suppose the path of `p` is '.l0.ggplot'
+#' q <- l_getFromPath('.l0.ggplot')
+#' # q is a `loon` widget
+#' q
+#' }
+#'
+#' # An alternative way to return a real loon widget from `p` (a `lggplot` object)
+#' # is to call the function `loon.ggplot`. Compared with calling function `l_getFrompath`
+#' # this way can provide richer information (note that it will create a new widget).
+#' q <- loon.ggplot(p)
+#' q
+#'
+#' # pipe more components
+#' p +
+#'   facet_grid(rows = vars(drv)) +
+#'   linking(linkingGroup = "mpg") +
+#'   ggtitle("displ versus cty")
+#' # a linked bar plot
+#' l_hist(mpg$class, linkingGroup = "mpg")
+#'
+#' # a 3D object
+#' # press the button key `R` to rotate the plot
+#' l_ggplot(mtcars,
+#'          mapping = aes(x = wt, y = hp, z = drat)) +
+#'    geom_point(size = 4)
 #' }
 l_ggplot <- function(data = NULL, mapping = aes(), ...,
                      environment = parent.frame()) {
@@ -50,29 +75,8 @@ l_ggplot <- function(data = NULL, mapping = aes(), ...,
 #' @export
 print.lggplot <- function(x, ...) {
 
-  params <- list()
-  if(!is.null(x$linking))
-    params <- x$linking$check_linkingKey(data = x$data, params = x$linking$params)
-  if(!is.null(x$select))
-    params <- c(params, x$select$check_selected(data = x$data, params = x$select$params))
-  if(!is.null(x$itemLabel))
-    params <- c(params, x$itemLabel$check_itemLabel(data = x$data, params = x$itemLabel$params))
-
-  p <- do.call(
-    ggplot2loon,
-    c(list(ggObj = x), params)
-  )
-  invisible()
+  p <- loon.ggplot(x, ...)
+  invisible(p)
 }
 
-#' @title Automatically create a loon widget
-#' @export
-#' @description It is retired. See \code{\link{l_ggplot}}
-#' @inheritParams l_ggplot
-lggplot <- function(data = NULL, mapping = aes(), ...,
-                    environment = parent.frame()) {
-  warning("`lggplot()` is deprecated now. Please use `l_ggplot()`",
-          call. = FALSE)
-  l_ggplot(data = data, mapping = mapping, ...,
-           environment = environment)
-}
+
