@@ -26,16 +26,11 @@ loon2ggplot <- function(target, ...) {
   if(ggplot2::is.ggplot(target) || is.ggmatrix(target)) {
     error_info <- deparse(substitute(target))
     stop(
-      paste0(
-        "Target should be a loon object. ",
-        "Maybe you want to call `ggplot2loon(",
-        error_info,
-        ")`?\n",
-        "Or, just call `loon.ggplot(`",
-        error_info,
-        ")` for simplification."
-      ),
-      call. = FALSE
+    "Target should be a loon object. Maybe you want to call `ggplot2loon(",
+    error_info,
+    ")`? Or, just call `loon.ggplot(",
+    error_info,
+    ")` for simplification.", call. = FALSE
     )
   }
   UseMethod('loon2ggplot', target)
@@ -44,7 +39,6 @@ loon2ggplot <- function(target, ...) {
 #' @export
 #' @rdname loon2ggplot
 loon2ggplot.default <- function(target, ...) {
-  # stop('loon2ggplot default no valid inheritance')
   # TODO
   ggObj <- list(...)$ggObj
   ggObj
@@ -54,7 +48,7 @@ loon2ggplot.default <- function(target, ...) {
 #' @export
 loon2ggplot.l_plot <- function(target, ...) {
 
-  loon::l_isLoonWidget(target) || stop(target, " does not exist", call. = FALSE)
+  loon::l_isLoonWidget(target) || stop("target does not exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
   cartesian_gg(target = target,
                ggObj = loon2ggplot(rl))
@@ -64,7 +58,7 @@ loon2ggplot.l_plot <- function(target, ...) {
 #' @export
 loon2ggplot.l_hist <- function(target, ...) {
 
-  loon::l_isLoonWidget(target) || stop("widget does not seem to exist", call. = FALSE)
+  loon::l_isLoonWidget(target) || stop("target does not seem to exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
   cartesian_gg(target = target,
                ggObj = loon2ggplot(rl))
@@ -73,7 +67,7 @@ loon2ggplot.l_hist <- function(target, ...) {
 #' @export
 loon2ggplot.l_graph <- function(target, ...) {
 
-  loon::l_isLoonWidget(target) || stop("widget does not seem to exist", call. = FALSE)
+  loon::l_isLoonWidget(target) || stop("target does not seem to exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
   cartesian_gg(target = target,
                ggObj = loon2ggplot(rl))
@@ -83,7 +77,7 @@ loon2ggplot.l_graph <- function(target, ...) {
 #' @export
 loon2ggplot.l_plot3D <- function(target, ...) {
 
-  loon::l_isLoonWidget(target) || stop("widget does not seem to exist", call. = FALSE)
+  loon::l_isLoonWidget(target) || stop("target does not seem to exist", call. = FALSE)
   rl <- loon::l_create_handle(c(target, 'root'))
 
   axes_coords <- target["axesCoords"]
@@ -209,7 +203,7 @@ cartesian_gg <- function(target, ggObj) {
       panel.grid.minor = ggplot2::element_line(size = 0.5,
                                                linetype = 'solid',
                                                colour = as_hex6color(widget['guidelines'])),
-      panel.border = if(sum(margins) > 0)
+      panel.border = if(sum(margins, na.rm = TRUE) > 0)
         ggplot2::element_rect(colour = as_hex6color(widget['foreground']),
                               fill = NA,
                               size = 1) else ggplot2::element_blank(),
